"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Failure = exports.Success = exports.Result = void 0;
const definitions_1 = require("../i18n/definitions");
class Result {
    constructor(warnings) {
        this.warnings = warnings;
    }
    addWarnings(warnings) {
        if (warnings && warnings.length) {
            if (this.warnings) {
                this.warnings.push(...warnings);
            }
            else {
                this.warnings = warnings;
            }
        }
        return this;
    }
    static encase(inner, failure) {
        try {
            return new Success(inner());
        }
        catch (e) {
            return failure(e);
        }
    }
    static sequence(list) {
        var _a;
        const result = [];
        const warnings = [];
        for (const item of list) {
            if ((_a = item.warnings) === null || _a === void 0 ? void 0 : _a.length) {
                warnings.push(...item.warnings);
            }
            if (item.isFailure()) {
                return item.addWarnings(warnings);
            }
            result.push(item.value);
        }
        return new Success(result).addWarnings(warnings);
    }
    static sequenceWithAllFailures(list) {
        var _a;
        const result = [];
        const warnings = [];
        let failure;
        for (const item of list) {
            if ((_a = item.warnings) === null || _a === void 0 ? void 0 : _a.length) {
                warnings.push(...item.warnings);
            }
            if (item.isFailure()) {
                const currentFailure = item;
                if (failure === undefined) {
                    failure = currentFailure;
                }
                else {
                    failure.errors.push(...currentFailure.errors);
                }
            }
            else {
                result.push(item.value);
            }
        }
        if (failure === undefined) {
            return new Success(result).addWarnings(warnings);
        }
        else {
            return failure.addWarnings(warnings);
        }
    }
    static sequenceWithoutFailure(list) {
        var _a;
        const result = [];
        const warnings = [];
        for (const item of list) {
            if ((_a = item.warnings) === null || _a === void 0 ? void 0 : _a.length) {
                warnings.push(...item.warnings);
            }
            if (item.isFailure()) {
                warnings.push(...item.errors);
            }
            else {
                result.push(item.value);
            }
        }
        return new Success(result).addWarnings(warnings);
    }
    static first(list, condition) {
        for (const item of list) {
            const result = condition(item);
            if (result.isFailure()) {
                return result;
            }
            else if (result.value) {
                return (new Success(item)).addWarnings(result.warnings);
            }
        }
        return new Success(undefined);
    }
    observe(f) {
        f(this);
        return this;
    }
}
exports.Result = Result;
class Success extends Result {
    constructor(value, warnings) {
        super(warnings);
        this.value = value;
    }
    isFailure() {
        return false;
    }
    isSuccess() {
        return true;
    }
    mapSuccess(onSuccess) {
        return onSuccess(this.value).addWarnings(this.warnings);
    }
    mapFailure(onFailure) {
        return this;
    }
    mapBoth(onFailure, onSuccess) {
        return onSuccess(this.value).addWarnings(this.warnings);
    }
    chainSuccess(next) {
        return next(this.value).addWarnings(this.warnings);
    }
    chainFailure(next) {
        return this;
    }
    chainBoth(onFailure, onSuccess) {
        return onSuccess(this.value).addWarnings(this.warnings);
    }
    downgradeErrorsToWarnings(withErrorFlag) {
        if (withErrorFlag) {
            return this.mapSuccess(() => new Success(true));
        }
        return this;
    }
    getAllMessages() {
        var _a;
        return (_a = this.warnings) !== null && _a !== void 0 ? _a : [];
    }
}
exports.Success = Success;
class Failure extends Result {
    constructor(errors, warnings) {
        super(warnings);
        this.errors = errors;
    }
    static single(code, parameters, issuer, type = definitions_1.MessageType.ProcessError) {
        return new Failure([{
                code,
                parameters,
                issuer,
                type,
            }]);
    }
    isFailure() {
        return true;
    }
    isSuccess() {
        return false;
    }
    mapSuccess(_) {
        return this;
    }
    mapFailure(onFailure) {
        return onFailure(this.errors).addWarnings(this.warnings);
    }
    mapBoth(onFailure, onSuccess) {
        return onFailure(this.errors).addWarnings(this.warnings);
    }
    chainSuccess(next) {
        return this;
    }
    chainFailure(next) {
        return next(this.errors).addWarnings(this.warnings);
    }
    chainBoth(onFailure, onSuccess) {
        return onFailure(this.errors).addWarnings(this.warnings);
    }
    downgradeErrorsToWarnings(withErrorFlag) {
        return new Success(withErrorFlag !== null && withErrorFlag !== void 0 ? withErrorFlag : true).addWarnings(this.errors);
    }
    getAllMessages() {
        if (this.warnings === undefined) {
            return this.errors;
        }
        else {
            return [...this.errors, ...this.warnings];
        }
    }
}
exports.Failure = Failure;
